/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.symbol;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.WeakHashMap;

/**
 * This class represent a terminal symbol
 *
 * @version $Revision: 1.2 $ $Date: 2002/04/18 08:19:56 $
 */
public class TerminalSymbol implements Symbol, Serializable
{

	private static WeakHashMap _memberTable = new WeakHashMap();

  private static int _nextOrdinal = 0;
	private transient int _ordinal = _nextOrdinal++;

	private String _name = null;

	/**
	 * The private constructor for the symbol
	 *
	 * @param name Name of symbol
	 */
	private TerminalSymbol(String name)
	{
		super();
		_name = name;
	}

	/*
	 * Returns an enumeration of all possible instances of ReduceTyp
	 */
	/*public static Enumeration enumerate()
	{
		return _memberTable.elements();
	}*/

  /**
   * Returns the name of this symbol
   *
   * @return Name of symbol
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Indicates whenever his is symbol is a terminal symbol
   *
   * @return True, if this symbol is a terminal symbol
   */
  public boolean isTerminal()
  {
    return true;
  }
  
  /**
   * Indicates whenever his is symbol is a nonterminal symbol
   *
   * @return True, if this symbol is a nonterminal symbol
   */
  public boolean isNonTerminal()
  {
    return false;
  }
  
  /**
   * Indicates whenever his is symbol is a symbol
   * for an empty list
   *
   * @return True, if this symbol indicates an empty list
   */
  public boolean isEmpty()
  {
    return false;
  }
    
	/**
	 * Returns the String representation of this symbol
	 */
	public String toString()
	{
		return _name;
	}

	/**
	 * Returns a new symbol based on the given name.
	 * @param name Name
	 */
	public static TerminalSymbol valueOf(String name)
	{
		Object obj = null;

		if (name != null)
			obj = _memberTable.get(name);
		if (obj == null)
		{
      obj = new TerminalSymbol(name);
      _memberTable.put(name, obj);
		}
		return (TerminalSymbol) obj;
	}

  private void writeObject(ObjectOutputStream out) throws IOException 
  {
    out.defaultWriteObject();
    out.writeUTF(_name);
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException  
  {
    in.defaultReadObject();
    _name = in.readUTF();
  }

  /** 
   * Deals with the issue of uniqueness when we are dealing with more
   * than one VM by adding the read symbol to the hash table, if it
   * isn't already there.
   */
  public Object readResolve() throws ObjectStreamException 
  {
    if (!_memberTable.containsKey(_name))
      _memberTable.put(_name, this);
    return (TerminalSymbol) (_memberTable.get(_name));
  }
}
